// UDP_Service class implementation file

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "UDP_Service.h"


UDP_Service::UDP_Service(char *server_ip, int server_port)
{
  server = false;
  client = true;
  
  destination_ip = server_ip;
  destination_port = server_port;

  printf("\nInitializing SDL_net (sending)...");
  if (SDLNet_Init() < 0)
  {
    fprintf(stderr, "FAILED!\nSDLNet_Init: %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  //open socket on random port
  printf("Opening random communication socket for sending...");
  if (!(sd_send = SDLNet_UDP_Open(0)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_UDP_Open(0): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  //resolve server address
  resolve_server_address();
  
  //allocate memory for packets
  printf("Allocating memory for sending packets...");
  if (!(p_send = SDLNet_AllocPacket(512)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_AllocPacket(512): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");

}

UDP_Service::UDP_Service(int listen_port)
{
  server = true;
  client = false;

  if (listen_port != 0)
    printf("Accepting connections on port: %d\n", listen_port);
  
  else
  {
    listen_port = 4555;
    printf("\nNo port given...\nDefaulting to port 4555...\n");
  }
  
  printf("Initializing SDL_net (recieve)...");  
  if (SDLNet_Init() < 0)    //init sdl_net
  {
    fprintf(stderr, "FAILED!\nSDLNet_Init: %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  // open socket on port given
  
  printf("Opening port %d for incoming connections...", listen_port);
  if (!(sd_recieve = SDLNet_UDP_Open(listen_port)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_UDP_Open(port): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  // allocate space for the packet
  printf("Allocating memory for recieving packets...");
  if (!(p_recieve = SDLNet_AllocPacket(512)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_AllocPacket(512): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n\n");
}

UDP_Service::~UDP_Service()
{
  if (client == true)
  {
    printf("Shutting down sending client...\n");
    printf("Freeing allocated memory for packets...");
    SDLNet_FreePacket(p_send);
    printf("Done!\nExiting SDLNet (sending)...");
    SDLNet_Quit();
    printf("Done!\n");
  }
    
  if (server == true)
  {
    printf("Shutting down recieving client...\n");
    printf("Freeing allocated memory for packets...");        
    SDLNet_FreePacket(p_recieve);
    printf("Done!\nExiting SDLNet (recieving)...");
    SDLNet_Quit();
    printf("Done!\n");
  }
 
}

void UDP_Service::send_data(char *message)
{
   //fill and send the packet
    //(char *)p_send->data = message;
    p_send->data = (unsigned char*)message;

    //set destination host & port
    p_send->address.host = srvadd.host;
    p_send->address.port = srvadd.port;

    //calculate length of data string
    p_send->len = strlen((char *)p_send->data) + 1;
    
    //send the packet
    SDLNet_UDP_Send(sd_send, -1, p_send);  //sets the p->channel
    
}

int UDP_Service::recieve_data(void)
{
  if (SDLNet_UDP_Recv(sd_recieve, p_recieve))
  {
    printf("Packet incoming!\n");
    printf("\tChan: %d\n", p_recieve->channel);
    printf("\tData: %s\n", (char *)p_recieve->data);
    printf("\t Len: %d\n", p_recieve->len);
    printf("\tMax Len: %d\n", p_recieve->maxlen);
    printf("\tStatus: %d\n", p_recieve->status);
    printf("\tAddress: %x %x\n\n", p_recieve->address.host, p_recieve->address.port);
      
    //if they sent the quit command
    if (!strcmp((char *)p_recieve->data, "quit"))
    {
      system("clear");
      printf("\nServer recieved \"quit\" signal from client!\n");
      return 0;
    }
  }
  
  return 1;
}

void UDP_Service::resolve_server_address(void)
{
  
  printf("Resolving server address...");
  if (SDLNet_ResolveHost(&srvadd, destination_ip, destination_port))
  {
    fprintf(stderr, "FAILED!\nSDLNet_ResolveHost(%s %d): %s\n", destination_ip, destination_port, SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
}

int UDP_Service::resolve_server_address(char *new_dest_ip)
{
  destination_ip = new_dest_ip;
  
  printf("Resolving new server IP @ %s on port %d...", destination_ip, destination_port);
  if (SDLNet_ResolveHost(&srvadd, destination_ip, destination_port))
  {
    fprintf(stderr, "FAILED!\nSDLNet_ResolveHost(%s %d): %s\n", destination_ip, destination_port, SDLNet_GetError());
    return 0;
  }
  printf("Success!\n");
  
  return 1;
}

int UDP_Service::resolve_server_address(int new_dest_port)
{
  destination_port = new_dest_port;
  
  printf("Setting destination port %d @ %s...", destination_port, destination_ip);
  if (SDLNet_ResolveHost(&srvadd, destination_ip, destination_port))
  {
    fprintf(stderr, "FAILED!\nSDLNet_ResolveHost(%s %d): %s\n", destination_ip, destination_port, SDLNet_GetError());
    return 0;
  }
  printf("Success!\n");
  return 1;
}

void UDP_Service::show_dest_info(void) const
{
  printf("Sending packets to %s on port %d\n", destination_ip, destination_port);
}
